"""reeln-cli plugin for Meta platform integration (Facebook Live, Instagram, Threads)."""

from __future__ import annotations

__version__ = "0.9.0"

from reeln_meta_plugin.plugin import MetaPlugin

__all__ = ["MetaPlugin", "__version__"]
